<?php
namespace Teknisa\MAN\Controllers;

use Teknisa\MAN\Services\FiliaisService;
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\ParameterBag;

class Filiais extends CrudWithParameterBag {
    protected $dataSourceName = 'filiais';
    protected $dataSourceNametoMessToo = 'relEstrutura';
    protected $dataSourceNameToFind = 'filiaisJoin';
    protected $parameterBagColumns = array('NRORG');
    /**
     * @var FiliaisService
     */
    protected $filiaisService;

    public function __construct (Manager $dataSourceManager, FiliaisService $filiaisService, ParameterBag $parameterBag) {
        $this->filiaisService = $filiaisService;
        parent::__construct($dataSourceManager, $parameterBag);
    }

    public function salvar($row, $dataSourceName, $response) {
        try {
            $this->dataSourceManager->persist(new DataSet($dataSourceName, array($row)));
        } catch (\Exception $e) {
            
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        foreach($request->getDataSet()->getRows() as &$row) {
            foreach($row['FILILOJA'] as $fililoja){
                $cods = explode('_', $fililoja);
                $row['CDFILIAL'] = $cods[0];
                if(!isset($cods[1])){
                    $row['CDLOJA'] = null;
                } else if(isset($cods[1])){
                    $row['CDLOJA'] = $cods[1];
                }
                $row['DTINCLUSAO'] = date("d/m/Y H:i:s");
                $row['DTULTATU'] = date("d/m/Y H:i:s");

                if($row['CDLOJA'] == null) {
                    $insereFili = $this->filiaisService->validaFilial($row['CDTABPREC'], $row['CDFILIALTABPREC'], $row['CDFILIAL'], $row['NRORG']);
                    $insereLoja['valida'] = \Service\FiliaisService::INSERE_LOJA;
                    if($insereFili['valida'] == \Service\FiliaisService::FILIAL_INATIVA){
                        $row['__is_new'] = false;
                        $row['IDATIVO'] = 'S';
                        $row['NRRELESTRUTURATABEPRECO'] = $insereFili['NRRELESTRUTURATABEPRECO'];
                        $request->getDataSet()->setDataSourceName('relEstruturaAtu');
                        $this->dataSourceManager->persist($request->getDataSet());
                        $response->addDataSet(new DataSet('resp', array(true)));
                    } else{
                        $response->addDataSet(new DataSet('resp', array(false)));
                    }
                }
                if($row['CDLOJA'] != null){
                    $insereFili = $this->filiaisService->validaFilial($row['CDTABPREC'], $row['CDFILIALTABPREC'], $row['CDFILIAL'], $row['NRORG']);
                    $insereLoja = $this->filiaisService->validaLoja($row['CDTABPREC'], $row['CDFILIALTABPREC'], $row['CDFILIAL'], $row['CDLOJA'], $row['NRORG']);
                    if($insereFili['valida'] == \Service\FiliaisService::FILIAL_ATIVA && $insereLoja['valida'] == \Service\FiliaisService::INSERE_LOJA){
                        $insereFili['valida'] = \Service\FiliaisService::INSERE_FILIAL;
                    } else if($insereFili['valida'] == \Service\FiliaisService::INSERE_FILIAL && $insereLoja['valida'] == \Service\FiliaisService::INSERE_LOJA){
                        $row['__is_new'] = true;
                        $aux = $row['CDLOJA'];
                        $row['CDLOJA'] = null;
                        $row['NRRELESTRUTURATABEPRECO'] = $this->filiaisService->getNextId();
                        $previousDataSourceName = $this->dataSourceName;
                        $this->dataSourceName = $this->dataSourceNametoMessToo;
                        $request->getDataSet()->setDataSourceName($this->dataSourceName);
                        $this->dataSourceManager->persist($request->getDataSet());
                        $this->dataSourceName = $previousDataSourceName;
                        $row['CDLOJA'] = $aux;
                    } else if($insereFili['valida'] == \Service\FiliaisService::FILIAL_INATIVA && $insereLoja['valida'] == \Service\FiliaisService::INSERE_LOJA) {
                        $row['__is_new'] = false;
                        $row['IDATIVO'] = 'S';
                        $aux = $row['CDLOJA'];
                        $row['CDLOJA'] = null;
                        $row['NRRELESTRUTURATABEPRECO'] = $insereFili['NRRELESTRUTURATABEPRECO'];
                        $request->getDataSet()->setDataSourceName('relEstruturaAtu');
                        $this->dataSourceManager->persist($request->getDataSet());
                        $row['CDLOJA'] = $aux;
                        $insereFili['valida'] = \Service\FiliaisService::INSERE_FILIAL;
                    } else if($insereFili['valida'] == \Service\FiliaisService::FILIAL_INATIVA && $insereLoja['valida'] == \Service\FiliaisService::LOJA_INATIVA) {
                        $row['__is_new'] = false;
                        $row['IDATIVO'] = 'S';
                        $aux = $row['CDLOJA'];
                        $row['CDLOJA'] = null;
                        $row['NRRELESTRUTURATABEPRECO'] = $insereFili['NRRELESTRUTURATABEPRECO'];
                        $request->getDataSet()->setDataSourceName('relEstruturaAtu');
                        $this->dataSourceManager->persist($request->getDataSet());
                        $row['CDLOJA'] = $aux;
                        $row['NRRELESTRUTURATABEPRECO'] = $insereLoja['NRRELESTRUTURATABEPRECO'];
                        $this->dataSourceManager->persist($request->getDataSet());
                        $response->addDataSet(new DataSet('resp', array(true)));
                    } else if($insereFili['valida'] == \Service\FiliaisService::FILIAL_ATIVA && $insereLoja['valida'] == \Service\FiliaisService::LOJA_INATIVA) {
                        $row['__is_new'] = false;
                        $row['IDATIVO'] = 'S';
                        $row['NRRELESTRUTURATABEPRECO'] = $insereLoja['NRRELESTRUTURATABEPRECO'];
                        $request->getDataSet()->setDataSourceName('relEstruturaAtu');
                        $this->dataSourceManager->persist($request->getDataSet());
                        $response->addDataSet(new DataSet('resp', array(true)));
                    } else{
                        $response->addDataSet(new DataSet('resp', array(false)));
                    }
                }
                if($insereFili['valida'] == \Service\FiliaisService::INSERE_FILIAL && $insereLoja['valida'] == \Service\FiliaisService::INSERE_LOJA){
                    $row['__is_new'] = true;
                    $row['NRRELESTRUTURATABEPRECO'] = $this->filiaisService->getNextId();
                    $previousDataSourceName = $this->dataSourceName;
                    $this->dataSourceName = $this->dataSourceNametoMessToo;
                    $request->getDataSet()->setDataSourceName($this->dataSourceName);
                    $this->dataSourceManager->persist($request->getDataSet());
                    $this->dataSourceName = $previousDataSourceName;
                    $response->addDataSet(new DataSet('resp', array(true)));
                }
            }
        }
    }

    public function findJoin(DTO\Request\Filter $request, DTO\Response $response) {
        $previousDataSourceName = $this->dataSourceName;
        $this->dataSourceName = $this->dataSourceNameToFind;
        parent::find($request, $response);
        $this->dataSourceName = $previousDataSourceName;
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response) {
        foreach($request->getDataSet()->getRows() as &$row){
            $row['DTULTATU'] = date("d/m/Y H:i:s");
            if($row['CDLOJA'] === null) {
                $this->dataSourceManager->persist(new DataSet('relEstruturaAtu', array($row)));
                $lojas = $this->filiaisService->getLojasByFilial($row['CDTABPREC'], $row['CDFILIALTABPREC'], $row['CDFILIAL'], $row['NRORG']);
                if($lojas != 1){
                    foreach ($lojas as $loja) {
                        $row['CDLOJA'] = $loja['CDLOJA'];
                        $row['NRRELESTRUTURATABEPRECO'] = $loja['NRRELESTRUTURATABEPRECO'];
                        $this->dataSourceManager->persist(new DataSet('relEstruturaAtu', array($row)));
                    }
                }
            } else {
                $this->dataSourceManager->persist(new DataSet('relEstruturaAtu', array($row)));
            }       
        }
    }
}